package MT::Plugin::UnpublishdOnSaver;

use strict;
use base qw( MT::Plugin );
use MT;

use MT::Util qw( format_ts relative_date remove_html encode_html encode_js
    encode_url archive_file_for offset_time_list break_up_text first_n_words );
use MT::I18N qw( const wrap_text );

our $VERSION = '0.01';

my $plugin = __PACKAGE__->new({
    id => 'UnpublishdOnSaver',
    name => 'UnpublishdOnSaver',
    version => $VERSION,
    description => '<MT_TRANS phrase="UnpublishdOnSaver.">',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    l10n_class  => 'UnpublishdOnSaver::L10N',
});
MT->add_plugin( $plugin );

*MT::CMS::Entry::save = \&plugin_save;

sub plugin_save {
    my $app = shift;
    $app->validate_magic or return;

    $app->remove_preview_file;

    if ( $app->param('is_power_edit') ) {
        return $app->save_entries(@_);
    }
    my $author = $app->user;
    my $type = $app->param('_type') || 'entry';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $cat_class = $app->model( $class->container_type );

    my $perms = $app->permissions
        or return $app->permission_denied();

    return $app->errtrans('Invalid request.')
        if $app->param('class');

    my $id = $app->param('id');
    if ( !$id ) {
        if ( $type eq 'page' ) {
            return $app->permission_denied()
                unless $perms->can_do('create_new_page');
        }
        elsif ( $type eq 'entry' ) {
            return $app->permission_denied()
                unless $perms->can_do('create_new_entry');
        }
        else {
            return $app->errtrans("Invalid request.");
        }
    }
    else {
        if ( $type eq 'page' ) {
            return $app->permission_denied()
                unless $perms->can_do('edit_all_pages');
        }
    }

    # check for autosave
    if ( $app->param('_autosave') ) {
        return $app->autosave_object();
    }

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    my $blog    = MT::Blog->load($blog_id)
        or return $app->error(
        $app->translate( 'Cannot load blog #[_1].', $blog_id ) );

    my $archive_type;

    my ( $obj, $orig_obj, $orig_file );
    if ($id) {
        $obj = $class->load($id)
            || return $app->error(
            $app->translate( "No such [_1].", $class->class_label ) );
        return $app->error( $app->translate("Invalid parameter") )
            unless $obj->blog_id == $blog_id;
        if ( $type eq 'entry' ) {
            return $app->permission_denied()
                unless $perms->can_edit_entry( $obj, $author );
            return $app->permission_denied()
                if ( $obj->status ne $app->param('status') )
                && !( $perms->can_edit_entry( $obj, $author, 1 ) );
            $archive_type = 'Individual';
        }
        elsif ( $type eq 'page' ) {
            $archive_type = 'Page';
        }
        else {
            return $app->errtrans("Invalid request.");
        }
        $orig_obj = $obj->clone;
        $orig_file = archive_file_for( $orig_obj, $blog, $archive_type );
    }
    else {
        $obj      = $class->new;
        $orig_obj = $obj->clone;
    }

    my ( $primary_category_old, $categories_old );
    if ( $orig_obj->id ) {
        $primary_category_old = $orig_obj->category;
        $categories_old       = $orig_obj->categories;
    }
    my $status_old = $id ? $obj->status : 0;
    my $names = $obj->column_names;

    ## Get rid of category_id param, because we don't want to just set it
    ## in the Entry record; save it for later when we will set the Placement.
    my ( $cat_id, @add_cat ) = split /\s*,\s*/,
        ( $app->param('category_ids') || '' );
    $app->delete_param('category_id');
    if ($id) {
        ## Delete the author_id param (if present), because we don't want to
        ## change the existing author.
        $app->delete_param('author_id');
    }

    my %values = map { $_ => scalar $app->param($_) } @$names;
    delete $values{'id'} unless $app->param('id');
    ## Strip linefeed characters.
    for my $col (qw( text excerpt text_more keywords )) {
        $values{$col} =~ tr/\r//d if $values{$col};
    }
    $values{allow_comments} = 0
        if !defined( $values{allow_comments} )
        || $app->param('allow_comments') eq '';
    delete $values{week_number}
        if ( $app->param('week_number') || '' ) eq '';
    delete $values{basename}
        unless $perms->can_do("edit_${type}_basename");
    require MT::Entry;
    $values{status} = MT::Entry::FUTURE() if $app->param('scheduled');
    $obj->set_values( \%values );
    $obj->allow_pings(0)
        if !defined $app->param('allow_pings')
        || $app->param('allow_pings') eq '';
    my $ao_d = $app->param('authored_on_date');
    my $ao_t = $app->param('authored_on_time');
    my $uo_d = $app->param('unpublished_on_date');
    my $uo_t = $app->param('unpublished_on_time');

    if ( !$id ) {

        #  basename check for this new entry...
        if (   ( my $basename = $app->param('basename') )
            && !$app->param('basename_manual')
            && $type eq 'entry' )
        {
            my $exist = $class->exist(
                { blog_id => $blog_id, basename => $basename } );
            if ($exist) {
                $obj->basename( MT::Util::make_unique_basename($obj) );
            }
        }
    }

    # Cut a basename by blog settings
    if ( $app->param('basename_manual') ) {
        first_n_words( $obj->basename, $blog->basename_limit );
    }

    if ( $type eq 'page' ) {

        # -1 is a special id for identifying the 'root' folder
        $cat_id = 0 if $cat_id == -1;
        my $dup_it = $class->load_iter(
            {   blog_id  => $blog_id,
                basename => $obj->basename,
                class    => 'page',
                ( $id ? ( id => $id ) : () )
            },
            { ( $id ? ( not => { id => 1 } ) : () ) }
        );
        my $folder;
        $folder = MT::Folder->load($cat_id) if $cat_id;
        while ( my $p = $dup_it->() ) {
            my ( $dup_path, $org_path );

            if ( $app->config('BasenameCheckCompat') ) {
                my $p_folder = $p->folder;
                $dup_path
                    = defined $p_folder ? $p_folder->publish_path() : '';
                $org_path = defined $folder ? $folder->publish_path() : '';
            }
            else {
                $dup_path = $p->permalink;

                my $url = $blog->site_url || "";
                $url .= '/' unless $url =~ m!/$!;
                $org_path
                    = $url . archive_file_for( $obj, $blog, 'Page', $folder );
            }

            return $app->error(
                $app->translate(
                    "This basename has already been used. You should use an unique basename."
                )
            ) if ( $dup_path eq $org_path );
        }

    }

    if ( $type eq 'entry' ) {
        $obj->status( MT::Entry::HOLD() )
            if !$id
            && !$perms->can_do('publish_own_entry')
            && !$perms->can_do('publish_all_entry');
    }

    my $filter_result
        = $app->run_callbacks( 'cms_save_filter.' . $type, $app );

    if ( !$filter_result ) {
        my %param = ();
        $param{error}       = $app->errstr;
        $param{return_args} = $app->param('return_args');
        $app->param( 'reedit', 1 );
        return $app->forward( "view", \%param );
    }

    # check to make sure blog has site url and path defined.
    # otherwise, we can't publish a released entry
    if ( ( $obj->status || 0 ) != MT::Entry::HOLD() ) {
        if ( !$blog->site_path || !$blog->site_url ) {
            return $app->error(
                $app->translate(
                    "Your blog has not been configured with a site path and URL. You cannot publish entries until these are defined."
                )
            );
        }
    }

    my ( $previous_old, $next_old );
    if ( $perms->can_do("edit_${type}_authored_on") && ($ao_d) ) {
        my %param = ();
        my $ao    = $ao_d . ' ' . $ao_t;
        my $ts    = MT::Util::valid_date_time2ts($ao);
        if ( !$ts ) {
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Published on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
                $ao
            );
        }
        $param{return_args} = $app->param('return_args');
        return $app->forward( "view", \%param ) if $param{error};
        if ( $obj->authored_on ) {
            $previous_old = $obj->previous(1);
            $next_old     = $obj->next(1);
        }
        $obj->authored_on($ts);
    }
    if (   $perms->can_do("edit_${type}_unpublished_on")
        && $obj->status != MT::Entry::UNPUBLISH() )
    {
        if ( $uo_d || $uo_t ) {
            my %param = ();
            my $uo    = $uo_d . ' ' . $uo_t;
            my $ts    = MT::Util::valid_date_time2ts($uo);
            if ( !$ts ) {
                $param{error} = $app->translate(
                    "Invalid date '[_1]'; 'Unpublished on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
                    $uo
                );
            }
            require MT::DateTime;
#            unless ( $param{error} ) {
#                $param{error} = $app->translate(
#                    "Invalid date '[_1]'; 'Unpublished on' dates should be dates in the future.",
#                    $uo
#                    )
#                    if (
#                    MT::DateTime->compare(
#                        blog => $blog,
#                        a    => { value => time(), type => 'epoch' },
#                        b    => $ts
#                    ) > 0
#                    );
#            }
#            if ( !$param{error} && $obj->authored_on ) {
#                $param{error} = $app->translate(
#                    "Invalid date '[_1]'; 'Unpublished on' dates should be later than the corresponding 'Published on' date.",
#                    $uo
#                    )
#                    if (
#                    MT::DateTime->compare(
#                        blog => $blog,
#                        a    => $obj->authored_on,
#                        b    => $ts
#                    ) > 0
#                    );
#            }
            $param{show_input_unpublished_on} = 1 if $param{error};
            $param{return_args} = $app->param('return_args');
            return $app->forward( "view", \%param ) if $param{error};
            if ( $obj->unpublished_on ) {
                $previous_old = $obj->previous(1);
                $next_old     = $obj->next(1);
            }
            $obj->unpublished_on($ts);
        }
        else {
            $obj->unpublished_on(undef);
        }
    }
    my $is_new = $obj->id ? 0 : 1;

    MT::Util::translate_naughty_words($obj);

    $app->run_callbacks( 'cms_pre_save.' . $type, $app, $obj, $orig_obj )
        || return $app->error(
        $app->translate(
            "Saving [_1] failed: [_2]",
            $class->class_label, $app->errstr
        )
        );

    # Setting modified_by updates modified_on which we want to do before
    # a save but after pre_save callbacks fire.
    $obj->modified_by( $author->id ) unless $is_new;

    $obj->save
        or return $app->error(
        $app->translate(
            "Saving [_1] failed: [_2]",
            $class->class_label, $obj->errstr
        )
        );

    # Clear cache for site stats dashboard widget.
    if (!$id
        || ((   ( $obj->status || 0 ) == MT::Entry::RELEASE()
                || $status_old eq MT::Entry::RELEASE()
            )
            && $status_old != $obj->status
        )
        )
    {
        require MT::Util;
        MT::Util::clear_site_stats_widget_cache($blog_id)
            or return $app->errtrans('Removing stats cache failed.');
    }

    ## look if any assets have been included/removed from this entry
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids') || '';
    my @asset_ids  = split( ',', $include_asset_ids );
    my $obj_assets = ();
    my @obj_assets = MT::ObjectAsset->load(
        { object_ds => 'entry', object_id => $obj->id } );
    foreach my $obj_asset (@obj_assets) {
        my $asset_id = $obj_asset->asset_id;
        $obj_assets->{$asset_id} = 1;
    }
    my $seen = ();
    foreach my $asset_id (@asset_ids) {
        my $obj_asset = MT::ObjectAsset->load(
            {   asset_id  => $asset_id,
                object_ds => 'entry',
                object_id => $obj->id
            }
        );
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id($blog_id);
            $obj_asset->asset_id($asset_id);
            $obj_asset->object_ds('entry');
            $obj_asset->object_id( $obj->id );
            $obj_asset->save;
        }
        $seen->{$asset_id} = 1;
    }
    foreach my $asset_id ( keys %{$obj_assets} ) {
        unless ( $seen->{$asset_id} ) {
            my $obj_asset = MT::ObjectAsset->load(
                {   asset_id  => $asset_id,
                    object_ds => 'entry',
                    object_id => $obj->id
                }
            );
            $obj_asset->remove;
        }
    }

    my $error_string = MT::callback_errstr();

    my $placements_updated;
    my $primary_category;

    ## Now that the object is saved, we can be certain that it has an
    ## ID. So we can now add/update/remove the primary placement.
    require MT::Placement;
    my $place
        = MT::Placement->load( { entry_id => $obj->id, is_primary => 1 } );
    if ($cat_id) {
        unless ($place) {
            $place = MT::Placement->new;
            $place->entry_id( $obj->id );
            $place->blog_id( $obj->blog_id );
            $place->is_primary(1);
        }
        $place->category_id($cat_id);
        $place->save;
        $primary_category   = $cat_class->load($cat_id);
        $placements_updated = 1;
    }
    else {
        if ($place) {
            $place->remove;
            $obj->cache_property( 'category', undef, undef );
            $placements_updated = 1;
        }
    }

    # save secondary placements...
    my @place = MT::Placement->load(
        {   entry_id   => $obj->id,
            is_primary => 0
        }
    );
    for my $place (@place) {
        $place->remove;
        $placements_updated = 1;
    }
    my @add_cat_obj;
    for my $cat_id (@add_cat) {
        my $cat = $cat_class->load($cat_id);

        # blog_id sanity check
        next if !$cat || $cat->blog_id != $obj->blog_id;

        my $place = MT::Placement->new;
        $place->entry_id( $obj->id );
        $place->blog_id( $obj->blog_id );
        $place->is_primary(0);
        $place->category_id($cat_id);
        $place->save
            or return $app->error(
            $app->translate(
                "Saving placement failed: [_1]", $place->errstr
            )
            );
        $placements_updated = 1;
        push @add_cat_obj, $cat;
    }
    if ($placements_updated) {
        unshift @add_cat_obj, $primary_category if $primary_category;
        @add_cat_obj = sort { $a->label cmp $b->label } @add_cat_obj;
        $obj->cache_property( 'categories', undef, \@add_cat_obj );
    }

    $app->run_callbacks( 'cms_post_save.' . $type, $app, $obj, $orig_obj );

    # Delete old archive files.
    if ( $app->config('DeleteFilesAtRebuild') && $id ) {
        my $file = archive_file_for( $obj, $blog, $archive_type );
        if ( $file ne $orig_file || $obj->status != MT::Entry::RELEASE() ) {
            $app->publisher->remove_entry_archive_file(
                Entry       => $orig_obj,
                ArchiveType => $archive_type,
                Category    => $primary_category_old,
            );
        }
    }

    ## If the saved status is RELEASE, or if the *previous* status was
    ## RELEASE, then rebuild entry archives, indexes, and send the
    ## XML-RPC ping(s). Otherwise the status was and is HOLD, and we
    ## don't have to do anything.
    if ( ( $obj->status || 0 ) == MT::Entry::RELEASE()
        || $status_old == MT::Entry::RELEASE() )
    {
        # If there are no static pages, just rebuild indexes.
        if ( $blog->count_static_templates($archive_type) == 0
            || MT::Util->launch_background_tasks() )
        {
            my $res = MT::Util::start_background_task(
                sub {
                    $app->run_callbacks('pre_build');
                    $app->rebuild_entry(
                        Entry => $obj,
                        (   $obj->is_entry
                            ? ( BuildDependencies => 1 )
                            : ( BuildIndexes => 1 )
                        ),
                        ( $obj->is_entry ? ( OldEntry => $orig_obj ) : () ),
                        (   $obj->is_entry
                            ? ( OldPrevious => ($previous_old)
                                ? $previous_old->id
                                : undef
                                )
                            : ()
                        ),
                        (   $obj->is_entry
                            ? ( OldNext => ($next_old)
                                ? $next_old->id
                                : undef
                                )
                            : ()
                        ),
                    ) or return $app->publish_error();
                    $app->run_callbacks( 'rebuild', $blog );
                    $app->run_callbacks('post_build');
                    1;
                }
            );
            return unless $res;
            if ( MT->has_plugin('Trackback') ) {
                require Trackback::CMS::Entry;
                return Trackback::CMS::Entry::ping_continuation(
                    $app,
                    $obj, $blog,
                    OldStatus => $status_old,
                    IsNew     => $is_new,
                );
            }
            else {
                return _finish_rebuild( $app, $obj, $is_new );
            }
        }
        else {
            return $app->redirect(
                $app->uri(
                    'mode' => 'start_rebuild',
                    args   => {
                        blog_id    => $obj->blog_id,
                        'next'     => 0,
                        type       => 'entry-' . $obj->id,
                        entry_id   => $obj->id,
                        is_new     => $is_new,
                        old_status => $status_old,
                        old_categories =>
                            join( ',', map { $_->id } @$categories_old ),
                        (   $previous_old
                            ? ( old_previous => $previous_old->id )
                            : ()
                        ),
                        ( $next_old ? ( old_next => $next_old->id ) : () )
                    }
                )
            );
        }
    }
    _finish_rebuild( $app, $obj, !$id );
}

sub _finish_rebuild {
    my $app = shift;
    my ( $entry, $is_new ) = @_;
    $app->redirect(
        $app->uri(
            'mode' => 'view',
            args   => {
                '_type' => $entry->class,
                blog_id => $entry->blog_id,
                id      => $entry->id,
                ( $is_new ? ( saved_added => 1 ) : ( saved_changes => 1 ) )
            }
        )
    );
}

1;
